//
//  bt_att_db.c
//
//  Bluetooth Protocol Stack - Attribute Database manager
//  Copyright (C) 2013-2014 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Update history
//	---------- ----- -------------------------------------------
//	2013.04.25 v0.0  First cording
//	2014.01.18 v0.0  WHM0.8
//
#include <stddef.h>
#include <string.h>
#include "bt_att_db.h"
#include "bt_spec_gatt.h"


//
//	ATT Database
//

#define INIT_UU16(a)	ATT_INITUUID16TO128(a)	// Alias
#define INIT_UU128(a)	ATT_INITUUID128(a)		// Alias
#define INIT_HANDLE(a)	ATT_INIT16(a)			// Alias

struct {
	uint8_t					entryNum;
	struct bt_attDbEntry	entry[];
}	bt_attDb = {
	83,
	{//	Handle	Type																						Len	 Values																															Property
		{0x0001,{INIT_UU16(GATT_Attribute_Types_Primary_Service)},											2,	{ATT_INIT16(GATT_Services_Generic_Access_Profile)},																				ATT_DB_PROPERTIES_READ},
		{0x0002, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										5,	{GATT_Characteristic_Properties_Read, INIT_HANDLE(0x0003), ATT_INIT16(GATT_Characteristic_Types_Device_Name)},					ATT_DB_PROPERTIES_READ},
		{0x0003, 		{INIT_UU16(GATT_Characteristic_Types_Device_Name)},									13,	{'W', 'H', ' ', 'M', 'e', 't', 'e', 'r', ' ', 'V', '0', '.', '8', '\0'},														ATT_DB_PROPERTIES_READ},
		{0x0004, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										5,	{GATT_Characteristic_Properties_Read, INIT_HANDLE(0x0005), ATT_INIT16(GATT_Characteristic_Types_Appearance)},					ATT_DB_PROPERTIES_READ},
		{0x0005, 		{INIT_UU16(GATT_Characteristic_Types_Appearance)},									2,	{ATT_INIT16(GATT_Characteristic_Appearance_Category_Generic_Computer)},															ATT_DB_PROPERTIES_READ},

		{0x0011,{INIT_UU16(GATT_Attribute_Types_Primary_Service)},											2,	{ATT_INIT16(GATT_Services_Generic_Attribute_Profile)},																			ATT_DB_PROPERTIES_READ},
		{0x0012, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										5,	{GATT_Characteristic_Properties_Read|GATT_Characteristic_Properties_Write_Without_Response|GATT_Characteristic_Properties_Indicate, ATT_INIT16(0x0013), ATT_INIT16(GATT_Characteristic_Types_Service_Changed)},	ATT_DB_PROPERTIES_READ},
		{0x0013, 		{INIT_UU16(GATT_Characteristic_Types_Service_Changed)},								4,	{ATT_INIT16(0x0001), ATT_INIT16(0xffff)},																						ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x0014, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x0015, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},

		{0x1000,{INIT_UU16(GATT_Attribute_Types_Primary_Service)},											16,	{INIT_UU128(UUID_MY_SERVICE)},																									ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		{0x1001, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										19,	{GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Write | GATT_Characteristic_Properties_Notify, INIT_HANDLE(0x1002), INIT_UU128(UUID_MY_CHAR0)},	ATT_DB_PROPERTIES_READ},
		{0x1002, 		{INIT_UU128(UUID_MY_CHAR0)},														7,	{ATT_INIT16(2013), 11, 4, 12, 34, 56},																							ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1003, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x1004, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1005, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},		8,	{'D', 'a', 't', 'e', 'T', 'i', 'm', 'e'},																						ATT_DB_PROPERTIES_READ},
		{0x1006, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},					7,	{GATT_Characteristic_Format_struct, 0, ATT_INIT16(GATT_Characteristic_Unit_time_second),
																												GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, ATT_INIT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		{0x1011, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										19,	{GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, INIT_HANDLE(0x1012), INIT_UU128(UUID_MY_CHAR1)},	ATT_DB_PROPERTIES_READ},
		{0x1012, 		{INIT_UU128(UUID_MY_CHAR1)},														20,	{ATT_INIT32(1),ATT_INIT32(2),ATT_INIT32(3),ATT_INIT32(4),ATT_INIT32(5)},														ATT_DB_PROPERTIES_READ},
		{0x1013, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x1014, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1015, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},		8,	{'C', 'H', '1', ' ', 'd', 'a', 't', 'a'},																						ATT_DB_PROPERTIES_READ},
		{0x1016, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},					7,	{GATT_Characteristic_Format_float32, 0, ATT_INIT16(GATT_Characteristic_Unit_power_watt),
																												GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, ATT_INIT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		{0x1021, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										19,	{GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, INIT_HANDLE(0x1022), INIT_UU128(UUID_MY_CHAR2)},	ATT_DB_PROPERTIES_READ},
		{0x1022, 		{INIT_UU128(UUID_MY_CHAR2)},														20,	{ATT_INIT32(1),ATT_INIT32(2),ATT_INIT32(3),ATT_INIT32(4),ATT_INIT32(5)},														ATT_DB_PROPERTIES_READ},
		{0x1023, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x1024, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1025, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},		8,	{'C', 'H', '2', ' ', 'd', 'a', 't', 'a'},																						ATT_DB_PROPERTIES_READ},
		{0x1026, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},					7,	{GATT_Characteristic_Format_float32, 0, ATT_INIT16(GATT_Characteristic_Unit_power_watt),
																												GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, ATT_INIT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		{0x1031, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										19,	{GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, INIT_HANDLE(0x1032), INIT_UU128(UUID_MY_CHAR3)},	ATT_DB_PROPERTIES_READ},
		{0x1032, 		{INIT_UU128(UUID_MY_CHAR3)},														20,	{ATT_INIT32(1),ATT_INIT32(2),ATT_INIT32(3),ATT_INIT32(4),ATT_INIT32(5)},														ATT_DB_PROPERTIES_READ},
		{0x1033, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x1034, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1035, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},		8,	{'C', 'H', '3', ' ', 'd', 'a', 't', 'a'},																						ATT_DB_PROPERTIES_READ},
		{0x1036, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},					7,	{GATT_Characteristic_Format_float32, 0, ATT_INIT16(GATT_Characteristic_Unit_power_watt),
																												GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, ATT_INIT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		{0x1041, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										19,	{GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, INIT_HANDLE(0x1042), INIT_UU128(UUID_MY_CHAR4)},	ATT_DB_PROPERTIES_READ},
		{0x1042, 		{INIT_UU128(UUID_MY_CHAR4)},														20,	{ATT_INIT32(1),ATT_INIT32(2),ATT_INIT32(3),ATT_INIT32(4),ATT_INIT32(5)},														ATT_DB_PROPERTIES_READ},
		{0x1043, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x1044, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1045, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},		8,	{'C', 'H', '4', ' ', 'd', 'a', 't', 'a'},																						ATT_DB_PROPERTIES_READ},
		{0x1046, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},					7,	{GATT_Characteristic_Format_float32, 0, ATT_INIT16(GATT_Characteristic_Unit_power_watt),
																												GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, ATT_INIT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		{0x1051, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										19,	{GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, INIT_HANDLE(0x1052), INIT_UU128(UUID_MY_CHAR5)},	ATT_DB_PROPERTIES_READ},
		{0x1052, 		{INIT_UU128(UUID_MY_CHAR5)},														20,	{ATT_INIT32(1),ATT_INIT32(2),ATT_INIT32(3),ATT_INIT32(4),ATT_INIT32(5)},														ATT_DB_PROPERTIES_READ},
		{0x1053, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x1054, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1055, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},		8,	{'C', 'H', '5', ' ', 'd', 'a', 't', 'a'},																						ATT_DB_PROPERTIES_READ},
		{0x1056, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},					7,	{GATT_Characteristic_Format_float32, 0, ATT_INIT16(GATT_Characteristic_Unit_power_watt),
																												GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, ATT_INIT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		{0x1061, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										19,	{GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, INIT_HANDLE(0x1062), INIT_UU128(UUID_MY_CHAR6)},	ATT_DB_PROPERTIES_READ},
		{0x1062, 		{INIT_UU128(UUID_MY_CHAR6)},														20,	{ATT_INIT32(1),ATT_INIT32(2),ATT_INIT32(3),ATT_INIT32(4),ATT_INIT32(5)},														ATT_DB_PROPERTIES_READ},
		{0x1063, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x1064, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1065, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},		8,	{'C', 'H', '6', ' ', 'd', 'a', 't', 'a'},																						ATT_DB_PROPERTIES_READ},
		{0x1066, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},					7,	{GATT_Characteristic_Format_float32, 0, ATT_INIT16(GATT_Characteristic_Unit_power_watt),
																												GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, ATT_INIT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		{0x1071, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										19,	{GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, INIT_HANDLE(0x1072), INIT_UU128(UUID_MY_CHAR7)},	ATT_DB_PROPERTIES_READ},
		{0x1072, 		{INIT_UU128(UUID_MY_CHAR7)},														20,	{ATT_INIT32(1),ATT_INIT32(2),ATT_INIT32(3),ATT_INIT32(4),ATT_INIT32(5)},														ATT_DB_PROPERTIES_READ},
		{0x1073, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x1074, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1075, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},		8,	{'C', 'H', '7', ' ', 'd', 'a', 't', 'a'},																						ATT_DB_PROPERTIES_READ},
		{0x1076, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},					7,	{GATT_Characteristic_Format_float32, 0, ATT_INIT16(GATT_Characteristic_Unit_power_watt),
																												GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, ATT_INIT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		{0x1101, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										19,	{GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Write | GATT_Characteristic_Properties_Notify, INIT_HANDLE(0x1102), INIT_UU128(UUID_MY_CHAR10)},	ATT_DB_PROPERTIES_READ},
		{0x1102, 		{INIT_UU128(UUID_MY_CHAR10)},														5,	{ATT_INIT16(2013),11,4,1},																						                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1103, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x1104, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1105, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},		7,	{'H', 'i', 's', 't', 'R', 'e', 'q'},																							ATT_DB_PROPERTIES_READ},
		{0x1106, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},					7,	{GATT_Characteristic_Format_struct, 0x00, ATT_INIT16(GATT_Characteristic_Unit_unitless),
																												GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, ATT_INIT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		{0x1111, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										19,	{GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Write | GATT_Characteristic_Properties_Notify, INIT_HANDLE(0x1112), INIT_UU128(UUID_MY_CHAR11)},	ATT_DB_PROPERTIES_READ},
		{0x1112, 		{INIT_UU128(UUID_MY_CHAR11)},														14,	{ATT_INIT16(1),ATT_INIT16(2),ATT_INIT16(3),ATT_INIT16(4),ATT_INIT16(5),ATT_INIT16(6),ATT_INIT16(7)},							ATT_DB_PROPERTIES_READ},
		{0x1113, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x1114, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1115, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},		7,	{'H', 'i', 's', 't', 'A', 'n', 's'},																							ATT_DB_PROPERTIES_READ},
		{0x1116, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},					7,	{GATT_Characteristic_Format_struct, 0, ATT_INIT16(GATT_Characteristic_Unit_unitless),
																												GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, ATT_INIT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		{0x1121, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										19,	{GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Write | GATT_Characteristic_Properties_Notify, INIT_HANDLE(0x1122), INIT_UU128(UUID_MY_CHAR12)},	ATT_DB_PROPERTIES_READ},
		{0x1122, 		{INIT_UU128(UUID_MY_CHAR12)},														20,	{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},																						ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1123, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x1124, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1125, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},		8,	{'S', 'e', 't', 'u', 'p', 'R', 'e', 'q'},																						ATT_DB_PROPERTIES_READ},
		{0x1126, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},					7,	{GATT_Characteristic_Format_struct, 0x00, ATT_INIT16(GATT_Characteristic_Unit_unitless),
																												GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, ATT_INIT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		{0x1131, 	{INIT_UU16(GATT_Attribute_Types_Characteristic)},										19,	{GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Write | GATT_Characteristic_Properties_Notify, INIT_HANDLE(0x1132), INIT_UU128(UUID_MY_CHAR13)},	ATT_DB_PROPERTIES_READ},
		{0x1132, 		{INIT_UU128(UUID_MY_CHAR13)},														20,	{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,},																						ATT_DB_PROPERTIES_READ},
		{0x1133, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},	2,	{ATT_INIT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},														ATT_DB_PROPERTIES_READ},
		{0x1134, 		{INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},	2,	{ATT_INIT16(0x0000)},																											ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
		{0x1135, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},		8,	{'S', 'e', 't', 'u', 'p', 'A', 'n', 's'},																						ATT_DB_PROPERTIES_READ},
		{0x1136, 		{INIT_UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},					7,	{GATT_Characteristic_Format_struct, 0, ATT_INIT16(GATT_Characteristic_Unit_unitless),
																												GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, ATT_INIT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},ATT_DB_PROPERTIES_READ},
	}
};

//
//	ATT DB access subroutines
//
											// Sequential read
struct bt_attDbEntry	*bt_attDbGetNext(	// OUT: Result data
		int		*id)						// IN,OUT: Access pointer (Start: *id = 0)
{
	struct bt_attDbEntry *p;

	if (bt_attDb.entryNum == 0) {
		p = NULL;
	} else {
		if (*id >= bt_attDb.entryNum) {
			p = NULL;
			*id = 0;
		} else {
			p = &bt_attDb.entry[*id];
			(*id)++;
		}
	}
	return(p);
}

										// Direct read
struct bt_attDbEntry	*bt_attDbFind(	// OUT: Result data
		unsigned short	startHandle,	// IN: Search key
		unsigned short	endHandle,		// IN: Search key
		unsigned char	*types,			// IN: Search key (UUID, NULL=All match)
		unsigned char	*value,			// IN: Search key (NULL=All match)
		size_t			valueLen)		// IN: "value" bytes
{
	struct bt_attDbEntry	*p;
	int						i;

	i = 0;
	while((p = bt_attDbGetNext(&i)) != NULL) {
		if (p->handle < startHandle) {
			continue;
		}
		if (p->handle > endHandle) {
			p = NULL;
			break;
		}
		if ((types != NULL) && memcmp(p->types, types, sizeof(p->types))) {
			continue;
		}
		if ((value != NULL) && memcmp(p->value, value, valueLen)) {
			continue;
		}
		break;
	}

	return(p);
}

										// Direct update
int bt_attDbUpdate(						// OUT: Return code (Negative value is error)
		unsigned short	handle,			// IN: Target
		unsigned char	*value,			// IN: Update data
		size_t			valueLen)		// IN: "value" bytes
{
	const unsigned char		clientConfig[] = {INIT_UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)};
	struct bt_attDbEntry	*p;

	if ((p = bt_attDbFind(handle, handle, NULL, NULL, 0)) != NULL) {
		if (valueLen > sizeof(p->value)) {
			valueLen = sizeof(p->value);
		}
		memcpy(p->value, value, valueLen);
		p->valueLen = valueLen;

		// User callback
		bt_attDbNotification(handle, value, valueLen);

		// Special DB record relation
		if (memcmp(clientConfig, p->types, sizeof(clientConfig)) == 0) {
			union bt_gatt_Attribute_Value	*v;
			int								c;
			unsigned short					h;

			v = (union bt_gatt_Attribute_Value	*)p->value;
			c = ATT_UINT16(v->Client_Characteristic_Configuration.Characteristic_Configuration_Bits);
			h = handle - 2;				// !! DB structure rule is this
			if ((p = bt_attDbFind(h, h, NULL, NULL, 0)) != NULL) {
				if (c & GATT_Client_Characteristic_Configuration_Notification) {
					p->properties |= ATT_DB_PROPERTIES_NOTIFY;
				} else {
					p->properties &= ~ATT_DB_PROPERTIES_NOTIFY;
				}
				if (c & GATT_Client_Characteristic_Configuration_Indication) {
					p->properties |= ATT_DB_PROPERTIES_INDICATE;
				} else {
					p->properties &= ~ATT_DB_PROPERTIES_INDICATE;
				}
			}
		}
	}
	return((p != NULL) ? 0 : -1);
}
