//
//  bt_att_db.h
//
//  Bluetooth Protocol Stack - Attribute Database manager
//  Copyright (C) 2013 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Update history
//	---------- ----- -------------------------------------------
//	2013.04.25 v0.0  First cording
//
#ifndef BT_ATT_DB_H_
#define BT_ATT_DB_H_

//
//	Data format
//
struct bt_attDbEntry {
	unsigned short	handle;		//
	unsigned char	types[16];	// UUID 128bits, bluetooth byte order
	size_t			valueLen;	//
	unsigned char	value[20];	// Single message size = 20
	unsigned char	properties;	//
};

#define ATT_DB_PROPERTIES_READ		0x02
#define ATT_DB_PROPERTIES_WRITE		0x08
#define ATT_DB_PROPERTIES_NOTIFY	0x10
#define ATT_DB_PROPERTIES_INDICATE	0x20

//
//	API
//
extern struct bt_attDbEntry	*bt_attDbGetNext(int *id);
extern struct bt_attDbEntry	*bt_attDbFind(unsigned short startHandle, unsigned short endHandle, unsigned char *types, unsigned char *value, size_t valueLen);
extern int					bt_attDbUpdate(unsigned short handle, unsigned char *value, size_t valueLen);
extern void					bt_attDbNotification(unsigned short handle, unsigned char *value, size_t valueLen);		// User routine

//
//	External parameter
//
// SERVICE-UUID
#define UUID_MY_SERVICE		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6c,0x00
// CHARACTERISTIC-UUID
#define UUID_MY_CHAR0		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x00
#define UUID_MY_CHAR1		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x01
#define UUID_MY_CHAR2		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x02
#define UUID_MY_CHAR3		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x03
#define UUID_MY_CHAR4		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x04
#define UUID_MY_CHAR5		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x05
#define UUID_MY_CHAR6		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x06
#define UUID_MY_CHAR7		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x07
#define UUID_MY_CHAR8		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x08
#define UUID_MY_CHAR9		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x09
#define UUID_MY_CHAR10		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x10
#define UUID_MY_CHAR11		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x11
#define UUID_MY_CHAR12		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x12
#define UUID_MY_CHAR13		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x13
#define UUID_MY_CHAR14		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x14
#define UUID_MY_CHAR15		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x15
#define UUID_MY_CHAR16		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x16
#define UUID_MY_CHAR17		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x17
#define UUID_MY_CHAR18		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x18
#define UUID_MY_CHAR19		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x19
#define UUID_MY_CHAR20		0xb6,0x1c,0xf1,0x93,0x79,0xb3,0x45,0x0a,0xbe,0x2f,0x16,0xd4,0x6e,0x21,0x6d,0x20

#endif /* BT_ATT_DB_H_ */
