//
//  s95160_s2.h	  Segment buffer support Layer2 (Hierarchy segment, Vertical relation, Touch action)
//  Graphic LCD module "SUCCESS ELECTRONIC S95160" I/O library
//  Copyright(C)2013-2014 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.12.26 v0.0  First cording
//    2013.12.27 v0.1  Release
//    2013.12.28 v0.1a Reducing the arguments
//    2013.12.31 v0.1b Scratch and build, value base -> refer base arguments
//    2014.01.04 v0.1b Splitting a source code (s2 -> s2+s3)
//

#ifndef S95160_S2_H_
#define S95160_S2_H_

#include "s95160_s1.h"

//
//	Configuration parameter
//
#define LCD_SEGMENT_TOUCH_THRESHOLD			30					// [%]
#define LCD_SEGMENT_TOUCH_CHATTER_CANCEL	100					// [event loop count]
#define LCD_SEGMENT_REDRAW_TIMEOUT			2000				// [SYSTICK] Chop the display refresh (option)

#ifdef LCD_SEGMENT_REDRAW_TIMEOUT
#define LCD_SEGMENT_SYSTICK_VARIABLE		systickTime			//
#include "systick.h"
#endif

//#define DEBUG


//
//	Control block
//
struct lcd_segmentGroup {			// Group > Figure, inclusive
	struct lcd_segmentFigure		fig;			// Essential item
	struct lcd_segmentArea			area;
	void							(*touchInProg)(struct lcd_segmentGroup *fig, struct lcd_segmentCoordinate *touch);
	void							(*touchOutProg)(struct lcd_segmentGroup *fig);
};

//
//	API
//
//extern struct lcd_segmentGroup*	lcd_segmentAllocGroup();
#define 							lcd_segmentAllocGroup()									((struct lcd_segmentGroup *)lcd_segmentAllocFigure(sizeof(struct lcd_segmentGroup)))
extern struct lcd_segment *			lcd_segmentNewGroup(struct lcd_segmentGroup *parentGroup, struct lcd_segmentGroup *myGroup);
extern void							lcd_segmentDeleteGroup(struct lcd_segmentGroup *fig);
extern struct lcd_segment *			lcd_segmentGetGroupToSegment(struct lcd_segmentGroup *fig);
extern struct lcd_segmentGroup *	lcd_segmentGetGroupFromSegment(struct lcd_segment *seg);
extern struct lcd_segmentGroup *	lcd_segmentGetGroupParent(struct lcd_segmentGroup *fig);
//extern void						lcd_segmentSetGroupDirty(struct lcd_segmentGroup *fig);
#define								lcd_segmentSetGroupDirty(fig)							lcd_segmentSetFigureDirty((struct lcd_segmentFigure *)(fig))
extern void							lcd_segmentSetGroupHidden(struct lcd_segmentGroup *fig, int f);
extern struct lcd_segmentGroup *	lcd_segmentFindGroupByPoint(struct lcd_segmentGroup *fig, struct lcd_segmentCoordinate *point);
extern void							lcd_segmentHandler(struct lcd_segmentGroup *rootGroup, struct lcd_segmentCoordinate *point);

//extern void						lcd_segmentDelete(void *fig);
#define								lcd_segmentDelete(fig)									lcd_segmentDeleteGroup((struct lcd_segmentGroup *)(fig))
//extern void						lcd_segmentSetDirty(void *fig);
#define								lcd_segmentSetDirty(fig)								lcd_segmentSetGroupDirty((struct lcd_segmentGroup *)(fig))
//extern void						lcd_segmentSetHidden(void *fig, int f);
#define								lcd_segmentSetHidden(fig, f)							lcd_segmentSetGroupHidden((struct lcd_segmentGroup *)(fig), f)

//
//	Internal routine
//
//extern int						lcd_segmentDisplayGroup(struct lcd_segmentFigure *fig, struct lcd_segmentCoordinate *cardinalPoint);

#endif /* S95160_S2_H_ */
