//
//  analog_capture.c	Free running A/D capture
//  Copyright(C)2013 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.05.23 v0.0  First cording
//
//

#include <stddef.h>
#include "stm32f4xx.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_adc.h"
#include "stm32f4xx_dma.h"
#include "stm32f4xx_tim.h"

#include "analog_capture.h"


void analogCaptureFreeRun(uint16_t buf[][CH_COUNT], int num, int usPeriod) {
	TIM_TimeBaseInitTypeDef	t;
	TIM_OCInitTypeDef		to;
	DMA_InitTypeDef			d;
	GPIO_InitTypeDef		g;
	ADC_InitTypeDef			a;
	ADC_CommonInitTypeDef	ac;

	//
	//	Clock
	//
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2 | RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB | RCC_AHB1Periph_GPIOC, ENABLE);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);

	//
	//	TIM3 Sampling Interval
	//
	t.TIM_Period        = usPeriod;										// 200 * 1us = 200us <- 1/50Hz/100sample=1/5000
	t.TIM_Prescaler     = ((SystemCoreClock / 2) / 1000000) - 1;	// 1MHz:1us
	t.TIM_ClockDivision = 0;
	t.TIM_CounterMode   = TIM_CounterMode_Up;
	TIM_InternalClockConfig(TIM3);
	TIM_TimeBaseInit(TIM3, &t);

	to.TIM_OCMode       = TIM_OCMode_PWM1;
	to.TIM_OutputState  = TIM_OutputState_Enable;
	to.TIM_Pulse        = 1;
	to.TIM_OCPolarity   = TIM_OCPolarity_High;
	TIM_OC1Init(TIM3, &to);
//	TIM_OC1PreloadConfig(TIM3, TIM_OCPreload_Disable);
	TIM_Cmd(TIM3, ENABLE);

	//
	//	DMA2 (Channel0-Stream0 = ADC1)
	//
	d.DMA_Channel            = DMA_Channel_0;
//	d.DMA_PeripheralBaseAddr = (uint32_t)ADC1_DR_ADDRESS;
	d.DMA_PeripheralBaseAddr = (uint32_t)&(ADC1->DR);
	d.DMA_Memory0BaseAddr    = (uint32_t)buf;
	d.DMA_DIR                = DMA_DIR_PeripheralToMemory;
	d.DMA_BufferSize         = CH_COUNT * num;
	d.DMA_PeripheralInc      = DMA_PeripheralInc_Disable;
	d.DMA_MemoryInc          = DMA_MemoryInc_Enable;
	d.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
	d.DMA_MemoryDataSize     = DMA_MemoryDataSize_HalfWord;
	d.DMA_Mode               = DMA_Mode_Circular;
	d.DMA_Priority           = DMA_Priority_High;
	d.DMA_FIFOMode           = DMA_FIFOMode_Disable;
	d.DMA_FIFOThreshold      = DMA_FIFOThreshold_HalfFull;
	d.DMA_MemoryBurst        = DMA_MemoryBurst_Single;
	d.DMA_PeripheralBurst    = DMA_PeripheralBurst_Single;
	DMA_Init(DMA2_Stream0, &d);
	DMA_Cmd(DMA2_Stream0, ENABLE);

	//
	//	ADC1 scan mode(CH1,2,3,8,9,14,15), trigger=TIM3
	//
	g.GPIO_Mode  = GPIO_Mode_AN;
//	g.GPIO_OType = GPIO_OType_PP;
	g.GPIO_PuPd  = GPIO_PuPd_NOPULL;
//	g.GPIO_Speed = GPIO_Speed_2MHz;
	g.GPIO_Pin   = GPIO_Pin_1 |GPIO_Pin_2 | GPIO_Pin_3;
	GPIO_Init(GPIOA, &g);
	g.GPIO_Pin   = GPIO_Pin_0 | GPIO_Pin_1;
	GPIO_Init(GPIOB, &g);
	g.GPIO_Pin   = GPIO_Pin_4 | GPIO_Pin_5;
	GPIO_Init(GPIOC, &g);

	ac.ADC_Mode             = ADC_Mode_Independent;
	ac.ADC_Prescaler        = ADC_Prescaler_Div4;			// Datasheet:ADC_CLK<36MHz, HCLK=168MHz/APB2_PRESC=2/ADC_CLK=36 -> 2.33, ADC_PRESC -> 4, ADC_CLK=21MHz
	ac.ADC_DMAAccessMode    = ADC_DMAAccessMode_Disabled;	// for dual/triple
	ac.ADC_TwoSamplingDelay = ADC_TwoSamplingDelay_5Cycles;	// for dual/triple
	ADC_CommonInit(&ac);

	a.ADC_Resolution           = ADC_Resolution_12b;
	a.ADC_ScanConvMode         = ENABLE;	// Multi channel
	a.ADC_ContinuousConvMode   = DISABLE;	// One shot
	a.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_Rising;
	a.ADC_ExternalTrigConv     = ADC_ExternalTrigConv_T3_CC1;
	a.ADC_DataAlign            = ADC_DataAlign_Right;
	a.ADC_NbrOfConversion      = 7;
	ADC_Init(ADC1, &a);

	ADC_RegularChannelConfig(ADC1, ADC_Channel_1,  1, ADC_SampleTime_56Cycles);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_2,  2, ADC_SampleTime_56Cycles);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_3,  3, ADC_SampleTime_56Cycles);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_8,  4, ADC_SampleTime_56Cycles);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_9,  5, ADC_SampleTime_56Cycles);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_14, 6, ADC_SampleTime_56Cycles);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_15, 7, ADC_SampleTime_56Cycles);
/*
	ADC_RegularChannelConfig(ADC1, ADC_Channel_1,  1, ADC_SampleTime_28Cycles);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_2,  2, ADC_SampleTime_28Cycles);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_3,  3, ADC_SampleTime_28Cycles);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_8,  4, ADC_SampleTime_28Cycles);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_9,  5, ADC_SampleTime_28Cycles);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_14, 6, ADC_SampleTime_28Cycles);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_15, 7, ADC_SampleTime_28Cycles);
*/
//	ADC_EOCOnEachRegularChannelCmd(ADC1, ENABLE);
	ADC_DMARequestAfterLastTransferCmd(ADC1, ENABLE);
	ADC_DMACmd(ADC1, ENABLE);
	ADC_Cmd(ADC1, ENABLE);

	return;
}

