//
//  sensor_data.h	WHM0.8 Control block
//  Copyright(C)2014 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2014.01.10 v0.0  main.c -> sensor_data.h
//
//

#ifndef SENSOR_DATA_H_
#define SENSOR_DATA_H_

//
//	Data structure
//

// Sensor setting and values table
struct sensorData {
	enum {
		notConnect = 0,
		ct,
		vsense
	}	infType;

	struct {
		struct currentSensor {
			float	delta;				// A/D Conversion parameter: y=ax+b: value=delta*ADC+offset
			float	offset;				//
			short	f;					// [Hz] Frequency
			short	v;					// [V] Now voltage
			short	a;					// [mA] Now current
			short	w;					// [W]  Now power
			long	ws_hur;				// [Ws] Used energy of this hour
			long	ws_day;				// [Ws] Used energy of this day
			long	ws_mon;				// [Ws] Used energy of this month
			long	ws_hurs[24];		// [Ws] Used energy of hour
			long	ws_days[32];		// [Ws] Used energy of day
			long	ws_mons[12];		// [Ws] Used energy of month
			struct __attribute__ ((packed)) currentSensorSetup {
				short			n;				// CT Turn count
				short			rl;				// CT Road register[ohm]
				float			k;				// CT Characteristic
				float			offset;			// Supplementary a revised
				short			f;				// Fixed frequency
				short			v;				// Fixed voltage
				unsigned char	vSenseChannel;	// >=CH_COUNT voltage sensing
				unsigned char	vSenseMulti;	// 1=x1.0 2=x2.0 ...
				char			name[8];
			}		setup;
		}	current;

		struct voltageSenseor {
			float	delta;				// A/D Conversion parameter: y=ax+b: value=delta*ADC+offset
			float	offset;				//
			short	f;					// [Hz] Frequency
			short	v;					// [V] Now voltage
			struct __attribute__ ((packed)) voltageSensorSetup {
				float			r1;		// Voltage sense register, adc side [ohm]
				float			r2;		// Voltage sense register, line side [ohm]
				float			offset;	// Supplementary a revised
			}		setup;
		}	voltage;
	}	inf;
};

extern struct sensorData	sensorData[CH_COUNT];


#endif /* SENSOR_DATA_H_ */
